#define KEY_BTAB 	FL_Tab
#define KEY_RIGHT 	FL_Right
#define KEY_LEFT	FL_Left
#define KEY_UP		FL_Up
#define KEY_DOWN	FL_Down
#define KEY_PPAGE	FL_Page_Up
#define KEY_NPAGE	FL_Page_Down
#define KEY_HOME	FL_Home
#define KEY_END		FL_End


#define KEY_F(k)			(FL_F+k)

#define KEY_BEG		0xffa0 /*?????????*/
#define KEY_LL		0xffa1 /*?????????*/
#define KEY_IC		FL_Insert
#define KEY_DC		FL_Delete
#define	KEY_A1		(FL_KP+'7')
#define	KEY_A3		(FL_KP+'9')
#define	KEY_C1		(FL_KP+'1')
#define	KEY_C3		(FL_KP+'3')
